import { ActionContext, Command, Middleware, Option } from "./types.js";
export interface IGroup {
    /**
     * Register a command
     * @param command - Command configuration
     * @returns void
     * @example
     * ```ts
     * group.command({
     *     name: 'my-command',
     *     description: 'Command example',
     *     options: [
     *         {
     *             name: '--my-option',
     *             description: 'Option example',
     *             alias: '-o'
     *         }
     *     ],
     *     action: (ctx) => {
     *         // ...
     *     }
     * });
     * ```
     */
    command: (command: Command) => IGroup;
    /**
     * Register multiple commands
     * @param commands - Commands to register
     * @returns void
     */
    commands: (...commands: Command[]) => IGroup;
    /**
     * Add a sub-group
     * @param name Name of the group
     * @param group Group object
     * @returns Igroup
     */
    group: (name: string, group: Group) => IGroup;
    /**
     * Register a middleware function
     * @param middleware Middleware function
     * @returns IGroup
     */
    use: (middleware: Middleware) => IGroup;
}
export interface GroupSpec {
    /**
     * Global options to be used for all commands in groups and subsequent groups
     */
    options?: Option[];
    /**
     * Description that shows up in the help menu for this group
     */
    description?: string;
}
export declare class Group implements IGroup {
    protected _groups: Map<string, Group>;
    protected _options: Option[];
    protected _commands: Command[];
    protected _description: string;
    protected _middlewares: Middleware[];
    protected _help(command?: Command): void;
    protected _run(input?: string[]): void;
    protected _runMiddleware(context: ActionContext): void;
    protected _validateCommand: (command: Command) => void;
    protected _validateGroup: (name: string, group: Group) => void;
    protected _validateOptions: (optionArray: Option[]) => void;
    constructor(spec?: GroupSpec);
    command(command: Command): this;
    commands(...commands: Command[]): this;
    group(name: string, group: Group): this;
    use(middleware: Middleware): this;
}
